<?php

namespace WebServiceCall\Call;

use Util\ArrayUtil;

use WebServiceCall\Log\FileLogger;
use WebServiceCall\Log\OutputStreamLogger;

use WebServiceCall\Service\Interfaces\IToken; 
use WebServiceCall\Service\Interfaces\ISales; 
use WebServiceCall\Service\Interfaces\IPayments;
use WebServiceCall\Service\Interfaces\IConciliacao;

use WebServiceCall\Service\Token as TokenService; 
use WebServiceCall\Service\Sales as SalesService; 
use WebServiceCall\Service\Payments as PaymentsService; 
use WebServiceCall\Service\Conciliacao as ConciliacaoService; 

class CncWS {
    private const LOG_DATE_FORMAT = "d/m/Y H:i:s";
    
    private $nrSeqControle;
    
    private $periodIni;
    private $periodEnd;
    
    private $configWS;
    
    public function __construct () {
        OutputStreamLogger::log(PHP_EOL . "Rotina iniciada em: " . date(self::LOG_DATE_FORMAT) . PHP_EOL);
        
        // Configurações gerais do WebService
        $this->configWS = new ConfigWS(); 
        
        // Busca os dados dos estabelecimentos vinculados.
        $connParams = $this->getConnParams(new TokenService());
        
        $initialDate = $this->configWS->getInitialDate();
        $finalDate   = $this->configWS->getFinalDate();
        
        $this->setSalesService(new SalesService ($connParams, $initialDate, $finalDate));
        $this->setPaymentsService(new PaymentsService ($connParams, $initialDate, $finalDate));
        $this->setConciliacaoService(new ConciliacaoService ($initialDate, $finalDate));
    }
    
    private function getConnParams(IToken $tokenService) {
        return $tokenService->getEstabelecCredentials();
    }
    
    private function setSalesService(ISales $sales) {
        $this->sales = $sales;
    }
    
    private function setPaymentsService(IPayments $payments) {
        $this->payments = $payments;
    }
    
    private function setConciliacaoService(IConciliacao $conciliacao) {
        $this->conciliacao = $conciliacao;
    }

    private function getValidEstabelecimentos() {
        $estabelecimentos = array_unique(array_merge(
            $this->sales->getEstabelecimentos(), 
            $this->payments->getEstabelecimentos()
        ));
        
        if ($this->areEstabelecimentosValid($estabelecimentos)) {
            throw new \Exception("Não foram encontrados estabelecimentos para a execução da rotina de conciliação.");
        }
        
        return $estabelecimentos;
    }
    
    private function areEstabelecimentosValid($estabelecimentos) {
        return count($estabelecimentos) === 0;
    }
    
    public function executeWS () {
        try {
            if($this->shouldPerformConciliacao()) {
                OutputStreamLogger::log(PHP_EOL . 'Importando Vendas');
                $this->sales->importSalesFromWebService();
                
                OutputStreamLogger::log(PHP_EOL . 'Importando Pagamentos');
                $this->payments->importPaymentsFromWebService();
              
                $estabelecimentos = $this->getValidEstabelecimentos();
                
                OutputStreamLogger::log(PHP_EOL . 'Realizando Conciliação');
                $this->conciliacao->performConciliacao($estabelecimentos);
            } else {
                throw new \Exception("Não foi possível realizar a conciliacao pois já houve uma replicação realizada neste período !");
            }
        }  catch (Throwable $e) {
            $str = PHP_EOL .'Fatal Error :' . PHP_EOL . $e->getMessage();
            FileLogger::log($str);
            OutputStreamLogger::log($str);
        }
        OutputStreamLogger::log(PHP_EOL . "Rotina finalizada em: " . date(self::LOG_DATE_FORMAT));
    }
    
    // Verifica se a base em questão já possuí dados replicados. 
    // No caso dessa ser a primeira execução do WS, o período que
    // será utilizado são os 30 últimos dias para fazer
    // o preenchimento inicial da base de produção.
    private function shouldPerformConciliacao()  {
        return $this->conciliacao->isConciliacaoPerformed();
    }
}