<?php 

namespace WebServiceCall\Service;

use WebServiceCall\Service\Interfaces\IEstabelecimento;

use WebServiceCall\Repository\DB\Estabelecimento as EstabelecimentoRepo;
use WebServiceCall\Repository\DB\Filial as FilialRepo;

class Estabelecimento implements IEstabelecimento {
    private const ESTABELECIMENTOS_NOT_FOUND = "Nenhum estabelecimento foi retornado para o código (CDESTABELEC) -> ";
    
    private $estabelecRepo;
    private $filialRepo;
    
    public function __construct() {
        $this->estabelecRepo = new EstabelecimentoRepo();
        $this->filialRepo = new FilialRepo();
    }
    
    public function checkEstabelecimentos($wsData) {
        $saleEstabelecs = array_column($wsData, ':Estabelecimento');
        
        // Se houver algum estabelecimento ainda não inserido na tabela de estabelecimento, eles devem ser inseridos
        // antes da inserção dos relatórios para que não ocorra erro de chave estrangeira não encontrada.
        foreach ($saleEstabelecs as $saleEstabelec) {
            $cdEstabelec = strval($saleEstabelec);
            
            if (!$this->doesEstabelecimentoExists($cdEstabelec)) {
                $this->estabelecRepo->insertEstabelecimento($cdEstabelec);        
                $this->estabelecRepo->commit();
            }
        }
        
        return $saleEstabelecs;
    }
    
    public function getNrEstabelecByCode($cdEstabelec) {
        // Troca o valor do Estabelecimento, substituindo o valor antigo que representava o código, para o nrestabelec
        // associado àquele código.
        $nrEstabelec = $this->estabelecRepo->getNrEstabelecByCode($cdEstabelec);
        if (!$nrEstabelec) {
            throw new \Exception(self::ESTABELECIMENTO_NOT_FOUND . $cdEstabelec);
        }
        return $nrEstabelec;
    }

    public function getNrEstabelecsByCodes(array $cdEstabelecs) {
        // Estabelecimentos must be an integer value, to remove to leading zeros.
        $mappedEstabelecs = array_map(function($cdEstabelec) {
            return intval($cdEstabelec);
        }, $cdEstabelecs);
        
        // Troca o valor do Estabelecimento, substituindo o valor antigo que representava o código, para o nrestabelec
        // associado àquele código.
        $nrEstabelecs = $this->estabelecRepo->getNrEstabelecsByCodes($mappedEstabelecs);
        if (!$nrEstabelecs) {
            throw new \Exception(self::ESTABELECIMENTOS_NOT_FOUND);
        }
        return $nrEstabelecs;
    }
    
    /*
     * Verifica se o CNPJ passado pertence a alguma filial já cadastrada. Se sim, atualiza os campos de filial para o estabelecimento
     * passado.
     *
     * @param cnpj: número do CNPJ da row atual do relatório
     * @param estabelec: número do estabelecimento da row atual do relatório
     */
    public function checkCNPJFilial($cnpj, $nrEstabelec) {
        $filialParams = $this->filialRepo->getFilialByCnpj($cnpj);
        
        if ($filialParams) {
            $this->estabelecRepo->updateEstabelecFilialParams($nrEstabelec, $cnpj, $filialParams);
            $this->estabelecRepo->commit();
        }
    }
    
    private function doesEstabelecimentoExists($cdEstabelec) {
        $result = $this->estabelecRepo->getEstabelecimentoByCode($cdEstabelec);
        return !empty($result);    
    }
}